@@ -37,6 +37,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/services"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/limitwriter"
 	"gitlab.com/gitlab-org/gitlab-runner/shells"
 )
 
@@ -51,6 +52,8 @@ const (
 	ExecutorStagePullingImage         common.ExecutorStage = "docker_pulling_image"
 )
 
+const ServiceLogOutputLimit = 64 * 1024
+
 var PrebuiltImagesPaths []string
 
 const (
@@ -1284,7 +1287,7 @@ func (e *executor) waitForServiceContainer(service *types.Container, timeout tim
 }
 
 func (e *executor) readContainerLogs(containerID string) string {
-	var containerBuffer bytes.Buffer
+	var buf bytes.Buffer
 
 	options := types.ContainerLogsOptions{
 		ShowStdout: true,
@@ -1298,7 +1301,10 @@ func (e *executor) readContainerLogs(containerID string) string {
 	}
 	defer func() { _ = hijacked.Close() }()
 
-	_, _ = stdcopy.StdCopy(&containerBuffer, &containerBuffer, hijacked)
-	containerLog := containerBuffer.String()
-	return strings.TrimSpace(containerLog)
+	// limit how much data we read from the container log to
+	// avoid memory exhaustion
+	w := limitwriter.New(&buf, ServiceLogOutputLimit)
+
+	_, _ = stdcopy.StdCopy(w, w, hijacked)
+	return strings.TrimSpace(buf.String())
 }
