@@ -76,7 +76,7 @@ func TestDockerWaiter_Wait(t *testing.T) {
 	}
 }
 
-func TestDockerWaiter_KillWait(t *testing.T) {
+func TestDockerWaiter_StopKillWait(t *testing.T) {
 	mClient := new(docker.MockClient)
 	defer mClient.AssertExpectations(t)
 
@@ -88,7 +88,7 @@ func TestDockerWaiter_KillWait(t *testing.T) {
 	var wg sync.WaitGroup
 
 	wg.Add(2)
-	mClient.On("ContainerKill", mock.Anything, mock.Anything, mock.Anything).
+	mClient.On("ContainerStop", mock.Anything, mock.Anything, mock.Anything).
 		Run(func(mock.Arguments) {
 			wg.Done()
 		}).
@@ -104,7 +104,7 @@ func TestDockerWaiter_KillWait(t *testing.T) {
 		}
 	}()
 
-	err := waiter.KillWait(context.Background(), "id")
+	err := waiter.StopKillWait(context.Background(), "id", nil)
 	assert.NoError(t, err)
 }
 
