@@ -42,7 +42,7 @@ const (
 
 	// The `.ps1` extension is added to the script name to fix a strange behavior
 	// where stage scripts wouldn't be executed otherwise
-	parsePwshScriptName = "parse_pwsh_script.ps1"
+	parsePwshScriptName = "parse_pwsh_script"
 
 	waitLogFileTimeout = time.Minute
 
@@ -64,7 +64,6 @@ var (
 		DefaultCustomBuildsDirEnabled: true,
 		DefaultBuildsDir:              "/builds",
 		DefaultCacheDir:               "/cache",
-		SharedBuildsDir:               true,
 		Shell: common.ShellScriptInfo{
 			Shell:         "bash",
 			Type:          common.NormalShell,
@@ -76,21 +75,6 @@ var (
 	errIncorrectShellType = fmt.Errorf("kubernetes executor incorrect shell type")
 )
 
-// GetDefaultCapDrop returns the default capabilities that should be dropped
-// from a build container.
-func GetDefaultCapDrop(os string) []string {
-	// windows does not support security context capabilities
-	if os == helperimage.OSTypeWindows {
-		return nil
-	}
-
-	return []string{
-		// Reasons for disabling NET_RAW by default were
-		// discussed in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/26833
-		"NET_RAW",
-	}
-}
-
 type commandTerminatedError struct {
 	exitCode int
 }
@@ -118,6 +102,16 @@ type kubernetesOptions struct {
 	Services common.Services
 }
 
+type containerBuildOpts struct {
+	name            string
+	image           string
+	imageDefinition common.Image
+	requests        api.ResourceList
+	limits          api.ResourceList
+	securityContext *api.SecurityContext
+	command         []string
+}
+
 type executor struct {
 	executors.AbstractExecutor
 
@@ -140,6 +134,8 @@ type executor struct {
 
 	remoteProcessTerminated chan shells.TrapCommandExitStatus
 
+	requireSharedBuildsDir *bool
+
 	// Flag if a repo mount and emptyDir volume are needed
 	requireDefaultBuildsDirVolume *bool
 }
@@ -164,6 +160,10 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) (err error) {
 
 	s.prepareOptions(options.Build)
 
+	// Dynamically configure use of shared build dir allowing
+	// for static build dir when isolated volume is in use.
+	s.SharedBuildsDir = s.isSharedBuildsDirRequired()
+
 	if err = s.checkDefaults(); err != nil {
 		return fmt.Errorf("check defaults error: %w", err)
 	}
@@ -394,19 +394,21 @@ func (s *executor) getContainerInfo(cmd common.ExecutorCommand) (string, []strin
 	var containerCommand []string
 	containerName := buildContainerName
 
-	switch s.Shell().Shell {
+	shell := s.Shell().Shell
+
+	switch shell {
 	case shells.SNPwsh, shells.SNPowershell:
 		// Translates to roughly "/path/to/parse_pwsh_script.ps1 /path/to/stage_script"
 		containerCommand = []string{
 			s.scriptPath(parsePwshScriptName),
 			s.scriptPath(cmd.Stage),
-			s.buildRedirectionCmd(),
+			s.buildRedirectionCmd(shell),
 		}
 		if cmd.Predefined {
 			containerName = helperContainerName
 			containerCommand = []string{fmt.Sprintf("Get-Content -Path %s | ", s.scriptPath(cmd.Stage))}
 			containerCommand = append(containerCommand, s.helperImageInfo.Cmd...)
-			containerCommand = append(containerCommand, s.buildRedirectionCmd())
+			containerCommand = append(containerCommand, s.buildRedirectionCmd(shell))
 		}
 	default:
 		// Translates to roughly "sh /detect/shell/path.sh /stage/script/path.sh"
@@ -416,7 +418,7 @@ func (s *executor) getContainerInfo(cmd common.ExecutorCommand) (string, []strin
 			"sh",
 			s.scriptPath(detectShellScriptName),
 			s.scriptPath(cmd.Stage),
-			s.buildRedirectionCmd(),
+			s.buildRedirectionCmd(shell),
 		}
 		if cmd.Predefined {
 			containerName = helperContainerName
@@ -427,7 +429,7 @@ func (s *executor) getContainerInfo(cmd common.ExecutorCommand) (string, []strin
 				s.helperImageInfo.Cmd,
 				"<<<",
 				s.scriptPath(cmd.Stage),
-				s.buildRedirectionCmd(),
+				s.buildRedirectionCmd(shell),
 			)
 		}
 	}
@@ -474,7 +476,7 @@ func (s *executor) buildPermissionsInitContainer(os string) (api.Container, erro
 			fmt.Sprintf(chmod, s.Build.RootDir),
 		}
 		container.Command = []string{
-			"pwsh",
+			s.Shell().Shell,
 			"-c",
 			strings.Join(commands, ";\n"),
 		}
@@ -491,7 +493,15 @@ func (s *executor) buildPermissionsInitContainer(os string) (api.Container, erro
 	return container, nil
 }
 
-func (s *executor) buildRedirectionCmd() string {
+func (s *executor) buildRedirectionCmd(shell string) string {
+	switch shell {
+	// powershell outputs utf16, so we re-encode the output to utf8
+	// this is important because our json decoder that detects the exit status
+	// of a job requires utf8.
+	case shells.SNPowershell:
+		return fmt.Sprintf("2>&1 | Out-File -Append -Encoding UTF8 %s", s.logFile())
+	}
+
 	return fmt.Sprintf("2>&1 | tee -a %s", s.logFile())
 }
 
@@ -596,9 +606,9 @@ func (s *executor) generateScripts(shell common.Shell) (map[string]string, error
 	shellName := s.Shell().Shell
 	switch shellName {
 	case shells.SNPwsh, shells.SNPowershell:
-		scripts[parsePwshScriptName] = shells.PwshValidationScript(shellName)
+		scripts[s.scriptName(parsePwshScriptName)] = shells.PwshValidationScript(shellName)
 	default:
-		scripts[detectShellScriptName] = shells.BashDetectShellScript
+		scripts[s.scriptName(detectShellScriptName)] = shells.BashDetectShellScript
 	}
 
 	for _, stage := range s.Build.BuildStages() {
@@ -609,7 +619,7 @@ func (s *executor) generateScripts(shell common.Shell) (map[string]string, error
 			return nil, fmt.Errorf("generating trap shell script: %w", err)
 		}
 
-		scripts[string(stage)] = script
+		scripts[s.scriptName(string(stage))] = script
 	}
 
 	return scripts, nil
@@ -643,7 +653,8 @@ func (s *executor) cleanupResources() {
 			CoreV1().
 			Pods(s.pod.Namespace).
 			Delete(context.TODO(), s.pod.Name, metav1.DeleteOptions{
-				PropagationPolicy: &PropagationPolicy,
+				GracePeriodSeconds: s.Config.Kubernetes.GetCleanupGracePeriodSeconds(),
+				PropagationPolicy:  &PropagationPolicy,
 			})
 		if err != nil {
 			s.Errorln(fmt.Sprintf("Error cleaning up pod: %s", err.Error()))
@@ -654,7 +665,9 @@ func (s *executor) cleanupResources() {
 		// TODO: handle the context properly with https://gitlab.com/gitlab-org/gitlab-runner/-/issues/27932
 		err := s.kubeClient.CoreV1().
 			Secrets(s.configurationOverwrites.namespace).
-			Delete(context.TODO(), s.credentials.Name, metav1.DeleteOptions{})
+			Delete(context.TODO(), s.credentials.Name, metav1.DeleteOptions{
+				GracePeriodSeconds: s.Config.Kubernetes.GetCleanupGracePeriodSeconds(),
+			})
 		if err != nil {
 			s.Errorln(fmt.Sprintf("Error cleaning up secrets: %s", err.Error()))
 		}
@@ -663,7 +676,9 @@ func (s *executor) cleanupResources() {
 		// TODO: handle the context properly with https://gitlab.com/gitlab-org/gitlab-runner/-/issues/27932
 		err := s.kubeClient.CoreV1().
 			ConfigMaps(s.configurationOverwrites.namespace).
-			Delete(context.TODO(), s.configMap.Name, metav1.DeleteOptions{})
+			Delete(context.TODO(), s.configMap.Name, metav1.DeleteOptions{
+				GracePeriodSeconds: s.Config.Kubernetes.GetCleanupGracePeriodSeconds(),
+			})
 		if err != nil {
 			s.Errorln(fmt.Sprintf("Error cleaning up configmap: %s", err.Error()))
 		}
@@ -671,12 +686,7 @@ func (s *executor) cleanupResources() {
 }
 
 //nolint:funlen
-func (s *executor) buildContainer(
-	name, image string,
-	imageDefinition common.Image,
-	requests, limits api.ResourceList,
-	containerCommand ...string,
-) (api.Container, error) {
+func (s *executor) buildContainer(opts containerBuildOpts) (api.Container, error) {
 	// check if the image/service is allowed
 	internalImages := []string{
 		s.Config.Kubernetes.Image,
@@ -687,16 +697,16 @@ func (s *executor) buildContainer(
 		optionName    string
 		allowedImages []string
 	)
-	if strings.HasPrefix(name, "svc-") {
+	if strings.HasPrefix(opts.name, "svc-") {
 		optionName = "services"
 		allowedImages = s.Config.Kubernetes.AllowedServices
-	} else if name == buildContainerName {
+	} else if opts.name == buildContainerName {
 		optionName = "images"
 		allowedImages = s.Config.Kubernetes.AllowedImages
 	}
 
 	verifyAllowedImageOptions := common.VerifyAllowedImageOptions{
-		Image:          image,
+		Image:          opts.image,
 		OptionName:     optionName,
 		AllowedImages:  allowedImages,
 		InternalImages: internalImages,
@@ -706,58 +716,50 @@ func (s *executor) buildContainer(
 		return api.Container{}, err
 	}
 
-	containerPorts := make([]api.ContainerPort, len(imageDefinition.Ports))
-	proxyPorts := make([]proxy.Port, len(imageDefinition.Ports))
+	containerPorts := make([]api.ContainerPort, len(opts.imageDefinition.Ports))
+	proxyPorts := make([]proxy.Port, len(opts.imageDefinition.Ports))
 
-	for i, port := range imageDefinition.Ports {
+	for i, port := range opts.imageDefinition.Ports {
 		proxyPorts[i] = proxy.Port{Name: port.Name, Number: port.Number, Protocol: port.Protocol}
 		containerPorts[i] = api.ContainerPort{ContainerPort: int32(port.Number)}
 	}
 
 	if len(proxyPorts) > 0 {
-		serviceName := imageDefinition.Alias
+		serviceName := opts.imageDefinition.Alias
 
 		if serviceName == "" {
-			serviceName = name
-			if name != buildContainerName {
-				serviceName = fmt.Sprintf("proxy-%s", name)
+			serviceName = opts.name
+			if opts.name != buildContainerName {
+				serviceName = fmt.Sprintf("proxy-%s", opts.name)
 			}
 		}
 
 		s.ProxyPool[serviceName] = s.newProxy(serviceName, proxyPorts)
 	}
 
-	pullPolicy, err := s.pullManager.GetPullPolicyFor(image)
+	pullPolicy, err := s.pullManager.GetPullPolicyFor(opts.image)
 	if err != nil {
 		return api.Container{}, err
 	}
 
-	command, args := s.getCommandAndArgs(imageDefinition, containerCommand...)
+	command, args := s.getCommandAndArgs(opts.imageDefinition, opts.command...)
 
 	container := api.Container{
-		Name:            name,
-		Image:           image,
+		Name:            opts.name,
+		Image:           opts.image,
 		ImagePullPolicy: pullPolicy,
 		Command:         command,
 		Args:            args,
 		Env:             buildVariables(s.Build.GetAllVariables().PublicOrInternal()),
 		Resources: api.ResourceRequirements{
-			Limits:   limits,
-			Requests: requests,
+			Limits:   opts.limits,
+			Requests: opts.requests,
 		},
-		Ports:        containerPorts,
-		VolumeMounts: s.getVolumeMounts(),
-		SecurityContext: &api.SecurityContext{
-			Privileged:               s.Config.Kubernetes.Privileged,
-			AllowPrivilegeEscalation: s.Config.Kubernetes.AllowPrivilegeEscalation,
-			Capabilities: getCapabilities(
-				GetDefaultCapDrop(s.helperImageInfo.OSType),
-				s.Config.Kubernetes.CapAdd,
-				s.Config.Kubernetes.CapDrop,
-			),
-		},
-		Lifecycle: s.prepareLifecycleHooks(),
-		Stdin:     true,
+		Ports:           containerPorts,
+		VolumeMounts:    s.getVolumeMounts(),
+		SecurityContext: opts.securityContext,
+		Lifecycle:       s.prepareLifecycleHooks(),
+		Stdin:           true,
 	}
 
 	return container, nil
@@ -789,7 +791,17 @@ func (s *executor) scriptsDir() string {
 }
 
 func (s *executor) scriptPath(stage common.BuildStage) string {
-	return path.Join(s.scriptsDir(), string(stage))
+	return path.Join(s.scriptsDir(), s.scriptName(string(stage)))
+}
+
+func (s *executor) scriptName(name string) string {
+	shell := s.Shell()
+	conf, err := common.GetShell(shell.Shell).GetConfiguration(*shell)
+	if err != nil || conf.Extension == "" {
+		return name
+	}
+
+	return name + "." + conf.Extension
 }
 
 func (s *executor) getVolumeMounts() []api.VolumeMount {
@@ -823,7 +835,7 @@ func (s *executor) getVolumeMounts() []api.VolumeMount {
 	if s.isDefaultBuildsDirVolumeRequired() {
 		mounts = append(mounts, api.VolumeMount{
 			Name:      "repo",
-			MountPath: s.Build.RootDir,
+			MountPath: s.AbstractExecutor.RootDir(),
 		})
 	}
 
@@ -999,16 +1011,24 @@ func (s *executor) getVolumesForSecrets() []api.Volume {
 func (s *executor) getVolumesForPVCs() []api.Volume {
 	var volumes []api.Volume
 
+	store := make(map[string]api.Volume)
+
 	for _, volume := range s.Config.Kubernetes.Volumes.PVCs {
-		volumes = append(volumes, api.Volume{
+		if _, found := store[volume.Name]; found {
+			continue
+		}
+
+		apiVolume := api.Volume{
 			Name: volume.Name,
 			VolumeSource: api.VolumeSource{
 				PersistentVolumeClaim: &api.PersistentVolumeClaimVolumeSource{
 					ClaimName: volume.Name,
-					ReadOnly:  volume.ReadOnly,
 				},
 			},
-		})
+		}
+
+		volumes = append(volumes, apiVolume)
+		store[volume.Name] = apiVolume
 	}
 
 	return volumes
@@ -1081,7 +1101,7 @@ func (s *executor) isDefaultBuildsDirVolumeRequired() bool {
 
 	var required = true
 	for _, mount := range s.getVolumeMountsForConfig() {
-		if mount.MountPath == s.Build.RootDir {
+		if mount.MountPath == s.AbstractExecutor.RootDir() {
 			required = false
 			break
 		}
@@ -1092,6 +1112,39 @@ func (s *executor) isDefaultBuildsDirVolumeRequired() bool {
 	return required
 }
 
+func (s *executor) isSharedBuildsDirRequired() bool {
+	// Return quickly when default builds dir is used as job is
+	// isolated to pod, so no need for SharedBuildsDir behavior
+	if s.isDefaultBuildsDirVolumeRequired() {
+		return false
+	}
+
+	var required = true
+	if s.requireSharedBuildsDir != nil {
+		return *s.requireSharedBuildsDir
+	}
+
+	// Fetch name of the volume backing the builds volume mount
+	buildVolumeName := "repo"
+	for _, mount := range s.getVolumeMountsForConfig() {
+		if mount.MountPath == s.AbstractExecutor.RootDir() {
+			buildVolumeName = mount.Name
+			break
+		}
+	}
+
+	// Require shared builds dir when builds dir volume is anything except an emptyDir
+	for _, volume := range s.getVolumes() {
+		if volume.Name == buildVolumeName && volume.VolumeSource.EmptyDir != nil {
+			required = false
+			break
+		}
+	}
+
+	s.requireSharedBuildsDir = &required
+	return required
+}
+
 func (s *executor) setupCredentials() error {
 	s.Debugln("Setting up secrets")
 
@@ -1158,13 +1211,17 @@ func (s *executor) setupBuildPod(initContainers []api.Container) error {
 	for i, service := range s.options.Services {
 		resolvedImage := s.Build.GetAllVariables().ExpandValue(service.Name)
 		var err error
-		podServices[i], err = s.buildContainer(
-			fmt.Sprintf("svc-%d", i),
-			resolvedImage,
-			service,
-			s.configurationOverwrites.serviceRequests,
-			s.configurationOverwrites.serviceLimits,
-		)
+		podServices[i], err = s.buildContainer(containerBuildOpts{
+			name:            fmt.Sprintf("svc-%d", i),
+			image:           resolvedImage,
+			imageDefinition: service,
+			requests:        s.configurationOverwrites.serviceRequests,
+			limits:          s.configurationOverwrites.serviceLimits,
+			securityContext: s.Config.Kubernetes.GetContainerSecurityContext(
+				s.Config.Kubernetes.ServiceContainerSecurityContext,
+				s.defaultCapDrop()...,
+			),
+		})
 		if err != nil {
 			return err
 		}
@@ -1229,6 +1286,20 @@ func (s *executor) setupBuildPod(initContainers []api.Container) error {
 	return nil
 }
 
+func (s *executor) defaultCapDrop() []string {
+	os := s.helperImageInfo.OSType
+	// windows does not support security context capabilities
+	if os == helperimage.OSTypeWindows {
+		return nil
+	}
+
+	return []string{
+		// Reasons for disabling NET_RAW by default were
+		// discussed in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/26833
+		"NET_RAW",
+	}
+}
+
 //nolint:funlen
 func (s *executor) preparePodConfig(
 	labels, annotations map[string]string,
@@ -1237,28 +1308,39 @@ func (s *executor) preparePodConfig(
 	hostAliases []api.HostAlias,
 	initContainers []api.Container,
 ) (api.Pod, error) {
-	buildImage := s.Build.GetAllVariables().ExpandValue(s.options.Image.Name)
+	dockerCmdForBuildContainer := s.BuildShell.DockerCommand
+	if s.Build.IsFeatureFlagOn(featureflags.KubernetesHonorEntrypoint) &&
+		!s.Build.IsFeatureFlagOn(featureflags.UseLegacyKubernetesExecutionStrategy) {
+		dockerCmdForBuildContainer = []string{}
+	}
 
-	buildContainer, err := s.buildContainer(
-		buildContainerName,
-		buildImage,
-		s.options.Image,
-		s.configurationOverwrites.buildRequests,
-		s.configurationOverwrites.buildLimits,
-		s.BuildShell.DockerCommand...,
-	)
+	buildContainer, err := s.buildContainer(containerBuildOpts{
+		name:            buildContainerName,
+		image:           s.Build.GetAllVariables().ExpandValue(s.options.Image.Name),
+		imageDefinition: s.options.Image,
+		requests:        s.configurationOverwrites.buildRequests,
+		limits:          s.configurationOverwrites.buildLimits,
+		securityContext: s.Config.Kubernetes.GetContainerSecurityContext(
+			s.Config.Kubernetes.BuildContainerSecurityContext,
+			s.defaultCapDrop()...,
+		),
+		command: dockerCmdForBuildContainer,
+	})
 	if err != nil {
 		return api.Pod{}, fmt.Errorf("building build container: %w", err)
 	}
 
-	helperContainer, err := s.buildContainer(
-		helperContainerName,
-		s.getHelperImage(),
-		common.Image{},
-		s.configurationOverwrites.helperRequests,
-		s.configurationOverwrites.helperLimits,
-		s.BuildShell.DockerCommand...,
-	)
+	helperContainer, err := s.buildContainer(containerBuildOpts{
+		name:     helperContainerName,
+		image:    s.getHelperImage(),
+		requests: s.configurationOverwrites.helperRequests,
+		limits:   s.configurationOverwrites.helperLimits,
+		securityContext: s.Config.Kubernetes.GetContainerSecurityContext(
+			s.Config.Kubernetes.HelperContainerSecurityContext,
+			s.defaultCapDrop()...,
+		),
+		command: s.BuildShell.DockerCommand,
+	})
 	if err != nil {
 		return api.Pod{}, fmt.Errorf("building helper container: %w", err)
 	}
@@ -1281,7 +1363,7 @@ func (s *executor) preparePodConfig(
 				buildContainer,
 				helperContainer,
 			}, services...),
-			TerminationGracePeriodSeconds: &s.Config.Kubernetes.TerminationGracePeriodSeconds,
+			TerminationGracePeriodSeconds: s.Config.Kubernetes.GetPodTerminationGracePeriodSeconds(),
 			ImagePullSecrets:              imagePullSecrets,
 			SecurityContext:               s.Config.Kubernetes.GetPodSecurityContext(),
 			HostAliases:                   hostAliases,
