@@ -12,6 +12,7 @@ import (
 	"net/url"
 	"os"
 	"regexp"
+	"strconv"
 	"strings"
 	"testing"
 	"time"
@@ -60,8 +61,9 @@ func TestRunIntegrationTestsWithFeatureFlag(t *testing.T) {
 		"testKubernetesReplaceEnvFeatureFlag":                     testKubernetesReplaceEnvFeatureFlag,
 		"testKubernetesReplaceMissingEnvVarFeatureFlag":           testKubernetesReplaceMissingEnvVarFeatureFlag,
 		"testKubernetesWithNonRootSecurityContext":                testKubernetesWithNonRootSecurityContext,
-		"testConfiguredBuildDirVolumeMountFeatureFlag":            testBuildDirVolumeMountFeatureFlag,
-		"testUserConfiguredBuildDirVolumeMountFeatureFlag":        testUserConfiguredBuildDirVolumeMountFeatureFlag,
+		"testBuildsDirDefaultVolumeFeatureFlag":                   testBuildsDirDefaultVolumeFeatureFlag,
+		"testBuildsDirVolumeMountEmptyDirFeatureFlag":             testBuildsDirVolumeMountEmptyDirFeatureFlag,
+		"testBuildsDirVolumeMountHostPathFeatureFlag":             testBuildsDirVolumeMountHostPathFeatureFlag,
 		"testKubernetesPwshFeatureFlag":                           testKubernetesPwshFeatureFlag,
 		"testKubernetesContainerHookFeatureFlag":                  testKubernetesContainerHookFeatureFlag,
 		"testKubernetesGarbageCollection":                         testKubernetesGarbageCollection,
@@ -111,6 +113,59 @@ func TestBuildScriptSections(t *testing.T) {
 	})
 }
 
+func TestEntrypointNotIgnored(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	testCases := map[string]struct {
+		useHonorEntrypointFF bool
+		expectedOutputLines  []string
+	}{
+		"feature flag off": {
+			useHonorEntrypointFF: false,
+			expectedOutputLines:  []string{"I am now root", "file not found"},
+		},
+		"feature flag on": {
+			useHonorEntrypointFF: true,
+			expectedOutputLines:  []string{"I am now nobody", "this has been executed through a custom entrypoint"},
+		},
+	}
+
+	for tn, tc := range testCases {
+		t.Run(tn, func(t *testing.T) {
+			build := getTestBuildWithImage(t, common.TestAlpineEntrypointImage, func() (common.JobResponse, error) {
+				job, err := common.GetRemoteBuildResponse(
+					"if [ -f /tmp/debug.log ]; then",
+					"cat /tmp/debug.log",
+					"else",
+					"echo 'file not found'",
+					"fi",
+					"echo \"I am now `whoami`\"",
+				)
+
+				job.Image = common.Image{
+					Name: common.TestAlpineEntrypointImage,
+				}
+
+				return job, err
+			})
+
+			build.Variables = append(
+				build.Variables,
+				common.JobVariable{Key: featureflags.KubernetesHonorEntrypoint, Value: strconv.FormatBool(tc.useHonorEntrypointFF)},
+			)
+
+			out, err := buildtest.RunBuildReturningOutput(t, build)
+			require.NoError(t, err)
+
+			t.Log(out)
+
+			for _, expectedLine := range tc.expectedOutputLines {
+				assert.Contains(t, out, expectedLine)
+			}
+		})
+	}
+}
+
 func testKubernetesMultistepRunFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
 
@@ -544,38 +599,32 @@ func testKubernetesReplaceMissingEnvVarFeatureFlag(t *testing.T, featureFlagName
 	assert.Contains(t, err.Error(), "image pull failed: Failed to apply default image tag \"alpine:\"")
 }
 
-func testBuildDirVolumeMountFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+func testBuildsDirDefaultVolumeFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
 
 	build := getTestBuild(t, common.GetRemoteSuccessfulBuild)
 	build.Image.Name = common.TestDockerGitImage
+	build.Runner.BuildsDir = "/path/to/builds/dir"
 	buildtest.SetBuildFeatureFlag(build, featureFlagName, featureFlagValue)
-	build.Runner.Kubernetes.Volumes = common.KubernetesVolumes{
-		EmptyDirs: []common.KubernetesEmptyDir{
-			{
-				Name:      "build",
-				MountPath: "/builds",
-				Medium:    "Memory",
-			},
-		},
-	}
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	assert.NoError(t, err)
+
+	assert.Equal(t, "/path/to/builds/dir/gitlab-org/ci-cd/tests/gitlab-test", build.BuildDir)
 }
 
-func testUserConfiguredBuildDirVolumeMountFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+func testBuildsDirVolumeMountEmptyDirFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
 
 	build := getTestBuild(t, common.GetRemoteSuccessfulBuild)
 	build.Image.Name = common.TestDockerGitImage
 	buildtest.SetBuildFeatureFlag(build, featureFlagName, featureFlagValue)
-	build.Runner.BuildsDir = "/some/path"
+	build.Runner.BuildsDir = "/path/to/builds/dir"
 	build.Runner.Kubernetes.Volumes = common.KubernetesVolumes{
 		EmptyDirs: []common.KubernetesEmptyDir{
 			{
-				Name:      "build",
-				MountPath: "/some/path",
+				Name:      "repo",
+				MountPath: "/path/to/builds/dir",
 				Medium:    "Memory",
 			},
 		},
@@ -583,6 +632,32 @@ func testUserConfiguredBuildDirVolumeMountFeatureFlag(t *testing.T, featureFlagN
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	assert.NoError(t, err)
+
+	assert.Equal(t, "/path/to/builds/dir/gitlab-org/ci-cd/tests/gitlab-test", build.BuildDir)
+}
+
+func testBuildsDirVolumeMountHostPathFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	build := getTestBuild(t, common.GetRemoteSuccessfulBuild)
+	build.Image.Name = common.TestDockerGitImage
+	buildtest.SetBuildFeatureFlag(build, featureFlagName, featureFlagValue)
+	build.Runner.Kubernetes.Volumes = common.KubernetesVolumes{
+		HostPaths: []common.KubernetesHostPath{
+			{
+				Name:      "repo-host",
+				MountPath: "/builds",
+				HostPath:  "/tmp/builds",
+			},
+		},
+	}
+
+	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
+	assert.NoError(t, err)
+
+	allVariables := build.GetAllVariables()
+
+	assert.Equal(t, fmt.Sprintf("/builds/%s/gitlab-org/ci-cd/tests/gitlab-test", allVariables.Get("CI_CONCURRENT_ID")), build.BuildDir)
 }
 
 // testKubernetesGarbageCollection tests the deletion of resources via garbage collector once the owning pod is deleted
@@ -1123,7 +1198,7 @@ func testKubernetesContainerHookFeatureFlag(t *testing.T, featureFlagName string
 	}
 }
 
-func getTestBuild(t *testing.T, getJobResponse func() (common.JobResponse, error)) *common.Build {
+func getTestBuildWithImage(t *testing.T, image string, getJobResponse func() (common.JobResponse, error)) *common.Build {
 	jobResponse, err := getJobResponse()
 	assert.NoError(t, err)
 
@@ -1136,7 +1211,7 @@ func getTestBuild(t *testing.T, getJobResponse func() (common.JobResponse, error
 			RunnerSettings: common.RunnerSettings{
 				Executor: "kubernetes",
 				Kubernetes: &common.KubernetesConfig{
-					Image:      common.TestAlpineImage,
+					Image:      image,
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 					PodLabels: map[string]string{
 						"test.k8s.gitlab.com/name": podUUID,
@@ -1145,6 +1220,11 @@ func getTestBuild(t *testing.T, getJobResponse func() (common.JobResponse, error
 			},
 		},
 	}
+
+}
+
+func getTestBuild(t *testing.T, getJobResponse func() (common.JobResponse, error)) *common.Build {
+	return getTestBuildWithImage(t, common.TestAlpineImage, getJobResponse)
 }
 
 func getTestKubeClusterClient(t *testing.T) *k8s.Clientset {
