@@ -16,7 +16,6 @@ import (
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/volume"
 	"github.com/docker/docker/client"
-	"github.com/docker/docker/errdefs"
 	"github.com/docker/docker/pkg/jsonmessage"
 	"github.com/docker/go-connections/tlsconfig"
 	"github.com/sirupsen/logrus"
@@ -39,21 +38,6 @@ func IsErrNotFound(err error) bool {
 	return client.IsErrNotFound(err)
 }
 
-// IsSystemError checks whether a returned error is due to an system error
-// Proxies the docker implementation.
-func IsSystemError(err error) bool {
-	for {
-		if errdefs.IsSystem(err) {
-			return true
-		}
-
-		err = errors.Unwrap(err)
-		if err == nil {
-			return false
-		}
-	}
-}
-
 // type officialDockerClient wraps a "github.com/docker/docker/client".Client,
 // giving it the methods it needs to satisfy the docker.Client interface
 type officialDockerClient struct {
@@ -153,6 +137,12 @@ func (c *officialDockerClient) ContainerKill(ctx context.Context, containerID st
 	return wrapError("ContainerKill", err, started)
 }
 
+func (c *officialDockerClient) ContainerStop(ctx context.Context, containerID string, timeout *time.Duration) error {
+	started := time.Now()
+	err := c.client.ContainerStop(ctx, containerID, timeout)
+	return wrapError("ContainerStop", err, started)
+}
+
 func (c *officialDockerClient) ContainerInspect(ctx context.Context, containerID string) (types.ContainerJSON, error) {
 	started := time.Now()
 	data, err := c.client.ContainerInspect(ctx, containerID)
